@@ -9,7 +9,7 @@ extern crate proc_macro;
 
 use proc_macro::TokenStream;
 
-use core::fmt;
+use core::{fmt, mem};
 
 #[cold]
 #[inline(never)]
@@ -17,7 +17,7 @@ fn compile_error<T: core::fmt::Display>(text: T) -> TokenStream {
     format!("core::compile_error!(\"{}\")", text).parse().unwrap()
 }
 
-enum Type {
+enum Primitive {
     U8,
     U16,
     U32,
@@ -25,17 +25,104 @@ enum Type {
     U128,
 }
 
+impl Primitive {
+    const fn size(&self) -> usize {
+        match self {
+            Primitive::U8 => mem::size_of::<u8>(),
+            Primitive::U16 => mem::size_of::<u16>(),
+            Primitive::U32 => mem::size_of::<u32>(),
+            Primitive::U64 => mem::size_of::<u64>(),
+            Primitive::U128 => mem::size_of::<u128>(),
+        }
+    }
+}
+
+enum Type {
+    Primitive(Primitive),
+    Array(Primitive, usize),
+}
+
 impl Type {
+    ///returns number of bytes written.
+    fn parse(input: &str) -> Result<Self, TokenStream> {
+        match input {
+            "" => Err(compile_error("'as' is missing type")),
+            "u8" => Ok(Type::Primitive(Primitive::U8)),
+            "u16" => Ok(Type::Primitive(Primitive::U16)),
+            "u32" => Ok(Type::Primitive(Primitive::U32)),
+            "u64" => Ok(Type::Primitive(Primitive::U64)),
+            "u128" => Ok(Type::Primitive(Primitive::U128)),
+            other => {
+                if let Some(arg) = input.strip_prefix('[') {
+                    if let Some(arg) = arg.strip_suffix(']') {
+                        let mut arg_split = arg.split(';');
+                        let arr_type = arg_split.next().unwrap();
+                        let arr_size = match arg_split.next() {
+                            Some(size) => size,
+                            None => return Err(compile_error(format_args!("'as' array expression '{}' is missing size", other))),
+                        };
+
+                        if let Some(_) = arg_split.next() {
+                            return Err(compile_error(format_args!("'as' array expression '{}' has superfluous ';'", other)));
+                        }
+                        let arr_type = match arr_type {
+                            "u8" => Primitive::U8,
+                            "u16" => Primitive::U16,
+                            "u32" => Primitive::U32,
+                            "u64" => Primitive::U64,
+                            "u128" => Primitive::U128,
+                            invalid => return Err(compile_error(format_args!("'as' array expression '{}' has invalid type '{}'", other, invalid))),
+                        };
+                        match arr_size.parse() {
+                            Ok(0) => Err(compile_error(format_args!("'as' array expression '{}' has zero size, which makes no sense", other))),
+                            Ok(arr_size) => Ok(Type::Array(arr_type, arr_size)),
+                            Err(err) => Err(compile_error(format_args!("'as' array expression '{}' has invalid size: {}", other, err))),
+                        }
+                    } else {
+                        Err(compile_error(format_args!("'as' array expression '{}' is missing closing brackets", other)))
+                    }
+                } else {
+                    Err(compile_error(format_args!("'as' specifies unsupported type '{}'", other)))
+                }
+            },
+        }
+    }
+
     ///returns number of bytes written.
     fn write_bytes<O: fmt::Write>(&self, out: &mut O, bytes: &[u8]) -> usize {
         match self {
-            Type::U8 => {
+            Type::Primitive(primitive) => primitive.write_bytes(out, bytes),
+            Type::Array(primitive, size) => {
+                let mut written = 0;
+                let required_size = primitive.size() * size;
+
+                if required_size > bytes.len() {
+                    return written;
+                }
+
+                for chunk in bytes.chunks_exact(required_size) {
+                    out.write_str("[").expect("to write string");
+                    written += primitive.write_bytes(out, chunk);
+                    out.write_str("],").expect("to write string");
+                }
+
+                written
+            },
+        }
+    }
+}
+
+impl Primitive {
+    ///returns number of bytes written.
+    fn write_bytes<O: fmt::Write>(&self, out: &mut O, bytes: &[u8]) -> usize {
+        match self {
+            Primitive::U8 => {
                 for byte in bytes {
                     core::fmt::write(out, format_args!("0x{:x}u8, ", byte)).expect("To write string");
                 }
                 bytes.len()
             },
-            Type::U16 => {
+            Primitive::U16 => {
                 let mut written = 0;
                 for chunk in bytes.chunks_exact(2) {
                     written += chunk.len();
@@ -44,7 +131,7 @@ impl Type {
                 }
                 written
             },
-            Type::U32 => {
+            Primitive::U32 => {
                 let mut written = 0;
                 for chunk in bytes.chunks_exact(4) {
                     written += chunk.len();
@@ -53,7 +140,7 @@ impl Type {
                 }
                 written
             }
-            Type::U64 => {
+            Primitive::U64 => {
                 let mut written = 0;
                 for chunk in bytes.chunks_exact(8) {
                     written += chunk.len();
@@ -62,7 +149,7 @@ impl Type {
                 }
                 written
             },
-            Type::U128 => {
+            Primitive::U128 => {
                 let mut written = 0;
                 for chunk in bytes.chunks_exact(16) {
                     written += chunk.len();
@@ -75,14 +162,23 @@ impl Type {
     }
 }
 
+impl fmt::Display for Primitive {
+    fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
+        match self {
+            Primitive::U8 => fmt.write_str("u8"),
+            Primitive::U16 => fmt.write_str("u16"),
+            Primitive::U32 => fmt.write_str("u32"),
+            Primitive::U64 => fmt.write_str("u64"),
+            Primitive::U128 => fmt.write_str("u128"),
+        }
+    }
+}
+
 impl fmt::Display for Type {
     fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
         match self {
-            Type::U8 => fmt.write_str("u8"),
-            Type::U16 => fmt.write_str("u16"),
-            Type::U32 => fmt.write_str("u32"),
-            Type::U64 => fmt.write_str("u64"),
-            Type::U128 => fmt.write_str("u128"),
+            Type::Primitive(primitive) => fmt::Display::fmt(primitive, fmt),
+            Type::Array(primitive, size) => fmt.write_fmt(format_args!("[{}; {}]", primitive, size)),
         }
     }
 }
@@ -106,20 +202,21 @@ impl<'a> Input<'a> {
             (file, &input[file.len()..])
         };
 
-        let mut split = input.trim().split_whitespace();
+        let input = input.trim();
+        let mut split = input.split_whitespace();
 
         let typ = match split.next() {
-            Some("as") => match split.next() {
-                None => return Err(compile_error("'as' is missing type")),
-                Some("u8") => Type::U8,
-                Some("u16") => Type::U16,
-                Some("u32") => Type::U32,
-                Some("u64") => Type::U64,
-                Some("u128") => Type::U128,
-                Some(other) => return Err(compile_error(format_args!("'as' specifies unsupported type '{}'", other))),
+            Some("as") => {
+                let arg = split.fold(String::new(), |mut acc, part| {
+                    acc.push_str(part);
+                    acc
+                });
+                let arg = arg.trim();
+
+                Type::parse(arg)?
             },
             Some(other) => return Err(compile_error(format_args!("Unsupported syntax after file name '{}'", other))),
-            None => Type::U8,
+            None => Type::Primitive(Primitive::U8),
         };
 
         Ok(Self {
@@ -148,9 +245,16 @@ impl<'a> Input<'a> {
 ///let bytes = include_bytes!("tests/include.in");
 ///let bytes_u16 = include_bytes!("tests/include.in" as u16);
 ///let bytes_u16_2 = include_bytes!("tests/include with whitespaces.in" as u16);
+///let bytes_u16_3 = include_bytes!("tests/include with whitespaces.in" as [u8; 48]);
+///let bytes_u16_4 = include_bytes!("tests/include with whitespaces.in" as [u16; 12]);
 ///
 ///assert_eq!(bytes.len(), bytes_u16.len() * 2);
 ///assert_eq!(bytes.len(), bytes_u16_2.len() * 2);
+///assert_eq!(bytes_u16_3.len(), 1);
+///assert_eq!(bytes_u16_3[0].len(), 48);
+///assert_eq!(bytes_u16_4.len(), 2);
+///assert_eq!(bytes_u16_4[0].len(), 12);
+///assert_eq!(bytes_u16_4[1].len(), 12);
 ///```
 pub fn include_bytes(input: TokenStream) -> TokenStream {
     let input = input.to_string();
@@ -188,10 +292,17 @@ pub fn include_bytes(input: TokenStream) -> TokenStream {
                 file_len += size;
                 let buf_len = cursor + size;
                 let written = args.typ.write_bytes(&mut result, &buf[..buf_len]);
-                unsafe {
-                    core::ptr::copy(buf.as_ptr().add(written), buf.as_mut_ptr(), buf_len - written);
+
+                if written > 0 {
+                    unsafe {
+                        core::ptr::copy(buf.as_ptr().add(written), buf.as_mut_ptr(), buf_len - written);
+                    }
+                    cursor = buf_len - written;
+                } else {
+                    //not enough data to write expression
+                    //wait another read.
+                    cursor += size;
                 }
-                cursor = buf_len - written;
             },
             Err(error) => {
                 return compile_error(format_args!("{}: Error reading file: {}", args.file, error))
